﻿using System;
using System.Text;
using System.Runtime.InteropServices;

namespace Field
{

#if COM_VISIBLE
    [ComVisible(true)]
    [Guid("E75F6471-3002-3321-B3C6-E8F6DC785DDC")]
    [ProgId("Field.Reports")]
    [ClassInterface(ClassInterfaceType.None)]
#endif
    public class Reports : IReports
    {
        // JSONのデータ型
        enum RP_TYPE {
            RP_TYPE_NULL = 0,
            RP_TYPE_BOOL = 1,
            RP_TYPE_INT = 2,
            RP_TYPE_FLOAT = 3,
            RP_TYPE_STRING = 4,
            RP_TYPE_ARRAY = 5,
            RP_TYPE_OBJECT = 6,
            RP_TYPE_OTHER = 9
        };

        // エラーの種類
        enum RP_ERROR {
            RP_NO_ERROR = 0,
            RP_REPORTS_ERROR = 1,
            RP_INPUT_ERROR = 2,
            RP_RENDER_ERROR = 3,
            RP_PDF_ERROR = 4,
            RP_AUTH_ERROR = 7,
            RP_RUNTIME_ERROR = 8,
            RP_SIG_ERROR = 9
        };

        // エラーメッセージの最大文字数
        const int RP_MAX_ERR_MSG_LEN = 1024;

        const string LibReports = "libreports";

        [DllImport(LibReports, CallingConvention = CallingConvention.Cdecl)]
        static extern unsafe void* rp_json_of_string(byte[] param);

        [DllImport(LibReports, CallingConvention = CallingConvention.Cdecl)]
        static extern void rp_init();

        [DllImport(LibReports, CallingConvention = CallingConvention.Cdecl)]
        static extern unsafe void rp_free(void* val);

        [DllImport(LibReports, CallingConvention = CallingConvention.Cdecl)]
        static extern unsafe bool rp_is_error(void* val);

        [DllImport(LibReports, CallingConvention = CallingConvention.Cdecl)]
        static extern unsafe IntPtr rp_get_error(void* val, int* err, byte[] msg);

        [DllImport(LibReports, CallingConvention = CallingConvention.Cdecl)]
        static extern unsafe uint rp_get_string_val(void* val, byte[] buf);

        [DllImport(LibReports, CallingConvention = CallingConvention.Cdecl)]
        static extern unsafe int rp_get_int_val(void* val);

        [DllImport(LibReports, CallingConvention = CallingConvention.Cdecl)]
        static extern unsafe void* rp_get_field(void* val, int index);

        [DllImport(LibReports, CallingConvention = CallingConvention.Cdecl)]
        static extern unsafe void* rp_version();

        [DllImport(LibReports, CallingConvention = CallingConvention.Cdecl)]
        static extern unsafe void* rp_set_log_level(int n);

        [DllImport(LibReports, CallingConvention = CallingConvention.Cdecl)]
        static extern unsafe void* rp_set_log_flag(int f);

        [DllImport(LibReports, CallingConvention = CallingConvention.Cdecl)]
        static extern unsafe void* rp_set_defaults(void* param);

        [DllImport(LibReports, CallingConvention = CallingConvention.Cdecl)]
        static extern unsafe void* rp_pdf_of_json(void* param);

        [DllImport(LibReports, CallingConvention = CallingConvention.Cdecl)]
        static extern unsafe void* rp_write_pdf_from_json(void* param, byte[] filename);

        [DllImport(LibReports, CallingConvention = CallingConvention.Cdecl)]
        static extern unsafe void* rp_signedpdf_of_json(void* param);

        [DllImport(LibReports, CallingConvention = CallingConvention.Cdecl)]
        static extern unsafe void* rp_write_signedpdf_from_json(void* param, byte[] filename);

        [DllImport(LibReports, CallingConvention = CallingConvention.Cdecl)]
        static extern unsafe void* rp_validatedpdf_of_json(void* param);

        [DllImport(LibReports, CallingConvention = CallingConvention.Cdecl)]
        static extern unsafe void* rp_write_validatedpdf_from_json(void* param, byte[] filename);

        static Reports() {
            rp_init();
        }

        private unsafe string GetErrorMessage(void* exn)
        {
            byte[] msg = new byte[RP_MAX_ERR_MSG_LEN];
            int err;
            rp_get_error(exn, &err, msg);
            rp_free(exn);
            return Encoding.UTF8.GetString(msg);
        }

        public unsafe void SetLogLevel(int n)
        {
            void* result = rp_set_log_level(n);
            if (rp_is_error(result)) {
                throw new Exception(GetErrorMessage(result));
            }
            rp_free(result);
        }

        public unsafe void SetLogFlag(int f)
        {
            void* result = rp_set_log_flag(f);
            if (rp_is_error(result)) {
                throw new Exception(GetErrorMessage(result));
            }
            rp_free(result);
        }

        public unsafe string Version()
        {
            void* caml_result = rp_version();
            if (rp_is_error(caml_result)) {
                throw new Exception(GetErrorMessage(caml_result));
            }
            byte[] buf = new byte[(int)rp_get_string_val(caml_result, null)];
            rp_get_string_val(caml_result, buf);
            rp_free(caml_result);
            return Encoding.UTF8.GetString(buf);
        }

        public unsafe void SetDefaults(string param)
        {
            Encoding utf8 = Encoding.UTF8;
            void* caml_param = rp_json_of_string(utf8.GetBytes(param + '\0'));
            if (rp_is_error(caml_param))
            {
                throw new Exception(GetErrorMessage(caml_param));
            }
            void* caml_result = rp_set_defaults(caml_param);
            if (rp_is_error(caml_result)) {
                rp_free(caml_param);
                throw new Exception(GetErrorMessage(caml_result));
            }
            rp_free(caml_param);
            rp_free(caml_result);
        }

        public unsafe byte[] Renders(string param)
        {
            Encoding utf8 = Encoding.UTF8;
            void* caml_param = rp_json_of_string(utf8.GetBytes(param + '\0'));
            if (rp_is_error(caml_param))
            {
                throw new Exception(GetErrorMessage(caml_param));
            }
            void* caml_result = rp_pdf_of_json(caml_param);
            if (rp_is_error(caml_result)) {
                rp_free(caml_param);
                throw new Exception(GetErrorMessage(caml_result));
            }
            byte[] buf = new byte[(int)rp_get_string_val(caml_result, null)];
            rp_get_string_val(caml_result, buf);
            rp_free(caml_param);
            rp_free(caml_result);
            return buf;
        }

        public unsafe void Render(string param, string filename)
        {
            Encoding utf8 = Encoding.UTF8;
            void* caml_param = rp_json_of_string(utf8.GetBytes(param + '\0'));
            if (rp_is_error(caml_param)) {
                throw new Exception(GetErrorMessage(caml_param));
            }
            void* caml_result = rp_write_pdf_from_json(caml_param, utf8.GetBytes(filename + '\0'));
            if (rp_is_error(caml_result)) {
                rp_free(caml_param);
                throw new Exception(GetErrorMessage(caml_result));
            }
            rp_free(caml_param);
            rp_free(caml_result);
        }

        public unsafe byte[] Signs(string param) {
            Encoding utf8 = Encoding.UTF8;
            void* caml_param = rp_json_of_string(utf8.GetBytes(param + '\0'));
            if (rp_is_error(caml_param))
            {
                throw new Exception(GetErrorMessage(caml_param));
            }
            void* caml_result = rp_signedpdf_of_json(caml_param);
            if (rp_is_error(caml_result)) {
                rp_free(caml_param);
                throw new Exception(GetErrorMessage(caml_result));
            }
            byte[] buf = new byte[(int)rp_get_string_val(caml_result, null)];
            rp_get_string_val(caml_result, buf);
            rp_free(caml_param);
            rp_free(caml_result);
            return buf;
        }

        public unsafe void Sign(string param, string filename) {
            Encoding utf8 = Encoding.UTF8;
            void* caml_param = rp_json_of_string(utf8.GetBytes(param + '\0'));
            if (rp_is_error(caml_param)) {
                throw new Exception(GetErrorMessage(caml_param));
            }
            void* caml_result = rp_write_signedpdf_from_json(caml_param, utf8.GetBytes(filename + '\0'));
            if (rp_is_error(caml_result)) {
                rp_free(caml_param);
                throw new Exception(GetErrorMessage(caml_result));
            }
            rp_free(caml_param);
            rp_free(caml_result);
        }

        public unsafe Tuple<string, byte[]> Validates(string param) {
            Encoding utf8 = Encoding.UTF8;
            void* caml_param = rp_json_of_string(utf8.GetBytes(param + '\0'));
            if (rp_is_error(caml_param)) {
                throw new Exception(GetErrorMessage(caml_param));
            }
            void* caml_result = rp_validatedpdf_of_json(caml_param);
            if (rp_is_error(caml_result)) {
                rp_free(caml_param);
                throw new Exception(GetErrorMessage(caml_result));
            }
            void* caml_validation = rp_get_field(caml_result, 0);
            byte[] validation = new byte[(int)rp_get_string_val(caml_validation, null)];
            rp_get_string_val(caml_validation, validation);
            void* caml_buf = rp_get_field(caml_result, 1);
            byte[] buf = new byte[(int)rp_get_string_val(caml_buf, null)];
            rp_get_string_val(caml_buf, buf);
            rp_free(caml_validation);
            rp_free(caml_buf);
            rp_free(caml_result);
            rp_free(caml_param);
            return Tuple.Create(System.Text.Encoding.ASCII.GetString(validation), buf);
        }

        public unsafe string Validate(string param, string filename) {
            Encoding utf8 = Encoding.UTF8;
            void* caml_param = rp_json_of_string(utf8.GetBytes(param + '\0'));
            if (rp_is_error(caml_param)) {
                throw new Exception(GetErrorMessage(caml_param));
            }
            void* caml_result = rp_write_validatedpdf_from_json(caml_param, utf8.GetBytes(filename + '\0'));
            if (rp_is_error(caml_result)) {
                rp_free(caml_param);
                throw new Exception(GetErrorMessage(caml_result));
            }
            byte[] result = new byte[(int)rp_get_string_val(caml_result, null)];
            rp_get_string_val(caml_result, result);
            rp_free(caml_param);
            rp_free(caml_result);
            return System.Text.Encoding.ASCII.GetString(result);
        }
    }
}
