#!/usr/bin/env python
# coding: utf-8

import unittest
import os
import sys
import json
import re

if hasattr(os, 'add_dll_directory'):
    for path in [path for path in re.split('[;:]', os.environ['PATH']) if os.path.exists(path)]:
        os.add_dll_directory(path)
from field import reports

class TestReports(unittest.TestCase):
    def test_version(self):
        self.assertTrue(reports.version())

    def test_renders1(self): 
        pdf = reports.renders({})
        self.assertTrue(len(pdf) > 100)
        self.assertEqual(pdf[:8], b'%PDF-1.6')

    def test_renders2(self): 
        pdf = reports.renders('{}')
        self.assertTrue(len(pdf) > 100)
        self.assertEqual(pdf[:8], b'%PDF-1.6')

    def test_render(self): 
        reports.render({}, 'out.pdf')
        pdf = open('out.pdf', 'rb').read()
        self.assertTrue(len(pdf) > 100)
        self.assertEqual(pdf[:8], b'%PDF-1.6')

    def tearDown(self):
        try:
            os.remove('out.pdf')
        except:
            pass

class TestParam(unittest.TestCase):
    def test_literal(self):
        src = {
            "int1": 123,
            "int2": 0,
            "int3": -123456,
            "float1": 123.456,
            "float2": -123.456,
            "str1": b"abcdefg",
            "str2": u"abcdefg",
            "str3": u"日本語".encode('utf-8'),
            "str4": u"日本語",
        }
        dst = json.loads(reports.set_defaults(src, debug=True))
        self.assertEqual(src["int1"], dst["int1"])
        self.assertEqual(src["int2"], dst["int2"])
        self.assertEqual(src["int3"], dst["int3"])
        self.assertEqual(src["float1"], dst["float1"])
        self.assertEqual(src["float2"], dst["float2"])
        self.assertEqual(src["str1"].decode('utf-8'), dst["str1"])
        self.assertEqual(src["str2"], dst["str2"])
        self.assertEqual(src["str3"].decode('utf-8'), dst["str3"])
        self.assertEqual(src["str4"], dst["str4"])

    def test_dict(self):
        src = {
            "L1" : {
                "L2": [ { "L3": 999 } ]
            }
        }
        dst = json.loads(reports.set_defaults(src, debug=True))
        self.assertEqual(src["L1"]["L2"][0]["L3"], 999)

def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestReports))
    suite.addTest(unittest.makeSuite(TestParam))
    return suite

if __name__ == '__main__':
    result = unittest.TextTestRunner(verbosity=2).run(suite())
    if not result.wasSuccessful():
        sys.exit(1)
