package jp.co.field_works;

import java.io.IOException;

/**
 * Reports.java
 *
 * Java VM bridge for Field Reports 
 * 
 * @author <a href="https://www.field-works.co.jp/">Field Works, LLC.</a>
 */
public class Reports {
    /**
     * 共有ライブラリのロードとモジュールの初期化
     */
    static {
        System.loadLibrary("JniReports");
        init();
    }

    /**
     * Reportsモジュールの初期化を行う。
     */
    private native static void init();
    
    /**
     * バージョン番号を取得します。
     */
    public native static String version()
        throws ReportsException;
    
    /**
     * ログ出力のレベルを設定します。
     * 有効な値の範囲は0〜4です:
     *  0: ログを出力しない
     *  1: ERRORログを出力する
     *  2: WARNログを出力する
     *  3: INFOログを出力する
     *  4: DEBUGログを出力する
     * 1以上の値を設定した場合，標準エラー出力にログを出力します（初期値：0）。
     */
    public native static void setLogLevel(int n)
        throws ReportsException;

    /**
     * レンダリング・パラメータのデフォルト値を設定します。
     * レンダリング・パラメータは，JSON形式の文字列で与えます。
     */
    public native static void setDefaults(String param)
        throws ReportsException;

    /**
     * レンダリング・パラメータparamを元にレンダリングを実行し，
     * 結果をバイト文字列として返します。
     * レンダリング・パラメータは，JSON形式の文字列で与えます。
     */
    public native static byte[] renders(String param)
        throws ReportsException;

    /**
     * レンダリング・パラメータparamを元にレンダリングを実行します。
     * 処理結果は，filenameで指定したファイルに出力されます。
     * レンダリング・パラメータは，JSON形式の文字列で与えます。
     */
    public native static void render(String param, String filename)
        throws ReportsException;

    /**
     * 署名パラメータparam（JSON文字列）を元に電子署名を実行し，
     * 署名済みPDFをバイト文字列として返します。
     */
    public native static byte[] signs(String param)
        throws ReportsException;

    /**
     * 署名パラメータparam（JSON文字列）を元に電子署名を実行します。
     * 署名済みPDFは，filenameで指定したファイルに出力されます。
     */
    public native static void sign(String param, String filename)
        throws ReportsException;

    /**
     * 署名パラメータparam（JSON文字列）を元に署名検証を実行します。
     * 検証結果をValidateResultとして返します。
     */
    public native static ValidateResult validates(String param)
        throws ReportsException;

    /**
     * 署名パラメータparam（JSON文字列）を元に署名検証を実行します。
     * 検証済みPDFはfilenameで指定したファイルに出力し，
     * 検証結果を文字列で返します。
     */
    public native static String validate(String param, String filename)
        throws ReportsException;

    public static void main(String[] args) throws ReportsException {
        try {
            // test
            //Reports.setLogLevel(3);
            System.out.println("version: " + Reports.version()); 
            System.out.write(Reports.renders("{}"));
        } catch (IOException e) {
            System.out.println("output error.");
        }
    }
}

